hledger-lib-1.51.1: A library providing the core functionality of hledger
Safe HaskellNone
LanguageGHC2021

Hledger.Data.BalanceData

Description

A 'BalanceData is a data type tracking a number of postings, exclusive, and inclusive balance for given date ranges.

Synopsis

Documentation

opBalanceData :: (MixedAmount -> MixedAmount -> MixedAmount) -> BalanceData -> BalanceData -> BalanceData Source #

Merge two BalanceData, using the given operation to combine their amounts.

Orphan instances