hledger-lib-1.51.1: A library providing the core functionality of hledger
Safe HaskellNone
LanguageGHC2021

Text.WideString

Description

Calculate the width of String and Text, being aware of wide characters.

Synopsis

Text Builders which keep track of length

data WideBuilder Source #

Helper for constructing Builders while keeping track of text width.

Constructors

WideBuilder 

Fields

wbUnpack :: WideBuilder -> String Source #

Convert a WideBuilder to a String.

wbToText :: WideBuilder -> Text Source #

Convert a WideBuilder to a strict Text.

wbFromText :: Text -> WideBuilder Source #

Convert a strict Text to a WideBuilder.